@@ -20,7 +20,7 @@
 #include <config.h>
 
 #define _GNU_SOURCE
-#ifdef linux
+#if defined(linux) || defined(__GLIBC__)
 /* For pread()/pwrite() */
 #define _XOPEN_SOURCE 500
 #endif
@@ -353,7 +353,6 @@ void backend_client_check()
 }
 
 #define MYTH_SEP "[]:[]"
-#define MYTH_PROGRAM_COLS  (mythtvfs.program_cols)
 
 int _version_check(int requested_version)
 {
@@ -385,9 +384,16 @@ struct backend_widths {
     int width;
 };
 
+int guess_program_cols(void);
+
 void backend_version_check()
 {
+    /*
+     * Version found in libs/libmythdb/mythversion.h MYTH_PROTO_VERSION
+     * Columns found in libs/libmyth/programinfo.h NUMPROGRAMLINES
+     */
     struct backend_widths versions[] = {
+        { 50, 47 },
         { 40, 46 },
         { 31, 42 },
         { 26, 41 },
@@ -423,8 +429,16 @@ void backend_version_check()
         }
 
         if (!found) {
-            fprintf(stderr,"Could not find compatible protocol version (mythtvfs=%d, backend=%d)\nPerhaps try with '-o program-width=N' where N is the new protocol width?\n",versions[0].version,version);
-            exit(1);
+            int guess = guess_program_cols();
+            if (guess > 0) {
+                mythtvfs.program_cols = guess;
+                found = 1;
+                fprintf(stderr,"Warning: unfamiliar backend version (%d), attempting guessed width %d.\n",version, mythtvfs.program_cols);
+            }
+            else {
+                fprintf(stderr,"Could not find compatible protocol version (mythtvfs=%d, backend=%d)\nPerhaps try with '-o program-width=N' where N is the new protocol width?\n",versions[0].version,version);
+                exit(1);
+            }
         }
     }
 
@@ -560,11 +574,11 @@ void clear_show(struct show_t * show)
     memset(show,0,sizeof(*show));
 }
 
-char * backend_pop_program(char * buffer, struct show_t * show)
+char * backend_pop_program(char * buffer, struct show_t * show, int col_width)
 {
     int col;
 
-    for (col = 0; buffer && col < MYTH_PROGRAM_COLS; col++) {
+    for (col = 0; buffer && col < col_width; col++) {
         char * sep = NULL;
         char * data = buffer;
         if ((sep = strstr(data,MYTH_SEP))!=NULL) {
@@ -575,7 +589,7 @@ char * backend_pop_program(char * buffer, struct show_t * show)
         // make sure we're not null
         if (!data) data="";
 
-        /* Found in libs/libmythtv/programinfo.cpp ProgramInfo::ToStringList */
+        /* Found in libs/libmyth/programinfo.cpp ProgramInfo::ToStringList */
         switch (col) {
             case 1: show->title = strdup(data); break;
             case 2: show->subtitle = strdup(data); break;
@@ -629,7 +643,14 @@ char * backend_pop_program(char * buffer, struct show_t * show)
             case 38: show->airdate = strdup(data); break;
             /*
             case 39: show->hasairdate = strdup(data); break;
-            case 40: show->playground = strdup(data); break;
+            case 40: show->playgroup = strdup(data); break;
+            case 41: show->recpriority = strdup(data); break;
+            case 42: show->parentid = strdup(data); break;
+            case 43: show->storagegroup = strdup(data); break;
+            case 44: show->audioprop = strdup(data); break;
+            case 45: show->videoprop = strdup(data); break;
+            case 46: show->subtitletype = strdup(data); break;
+            case 47: show->year = strdup(data); break;
             ...
             */
             default:
@@ -641,6 +662,49 @@ char * backend_pop_program(char * buffer, struct show_t * show)
     return buffer;
 }
 
+/* return guessed program column width, or -1 for failure */
+int guess_program_cols(void)
+{
+    /* Since most of the program data are either arbitrary numbers of
+       free-form text, we need to find something that stands out as
+       system-specific to calculate program column width.  This requires
+       that at least 2 recordings exist (to determine a delta), and that
+       the recording pathname is fully qualified (which makes the
+       assumption that other text strings do not start with "/").  Since
+       storage pools may be different, we cannot use more than the first
+       character (though different storage pools would break the current
+       mythtvfs anyway, since it assumes that all recordings are in the
+       same place).
+     */
+
+    char * buffer, * data;
+    int guessed = 0;
+    struct show_t show = { };
+
+    /* First, extract the pathname, which will be at col 9 */
+    data = buffer = backend_query();
+    data = backend_pop_program(data, &show, 10);
+    /* Make sure path has a leading slash */
+    if (!show.pathname || strncmp(show.pathname, "myth://", 7)!=0) return -1;
+
+    while (data) {
+        char * sep = NULL;
+
+        guessed ++;
+        if ((sep = strstr(data,MYTH_SEP))!=NULL) {
+            *sep = '\0';
+            sep+=strlen(MYTH_SEP);
+            if (strncmp(data, "myth://", 7) == 0) break;
+        }
+        data = sep;
+    }
+
+    if (!data) guessed = -1;
+    free(buffer);
+
+    return guessed;
+}
+
 void mark_stale(gpointer key, gpointer value, gpointer user_data)
 {
     struct mythtv_recording * recording = (struct mythtv_recording*)value;
@@ -691,7 +755,7 @@ void refresh_recordings()
     /* mark all hash items stale */
     g_hash_table_foreach(mythtvfs.recordings, mark_stale, NULL);
 
-    while ( (data = backend_pop_program(data,&show)) ) {
+    while ( (data = backend_pop_program(data,&show,mythtvfs.program_cols)) ) {
         char *metaname = NULL, *ptr = NULL;
 
 #ifdef MYTHTVFS_DEBUG_FUNC
